/*
 *  TrackRenderer — Render overlay tiles of GPS tracks for OpenLayers
 *  Copyright (C) 2008 Joerg Reuter <jreuter@yaina.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package de.yaina.trackrenderer;

/**
 * A tiled canvas with some supposedly higher-level drawing functions.
 * 
 * @author Joerg Reuter <jreuter@yaina.de>
 */
public class DrawCanvas extends TileCanvas {

    static TileCoordinates ps;

    /**
     * Create a new canvas to draw on.
     * 
     * @param tileDimX horizontal size of the tiles
     * @param tileDimY vertical size of the tiles
     */
    public DrawCanvas(int tileDimX, int tileDimY) {
        super(tileDimX, tileDimY);
        ps = new TileCoordinates(0, 0, 0, 0);
    }

    private TileCoordinates[] normalizePair(TileCoordinates p1, TileCoordinates p2) {
        TileCoordinates[] result = new TileCoordinates[2];
        int dimx = getDimensionX();
        int dimy = getDimensionY();

        /* p2 on different tile? Remap to p1's tile, super.setPoint() will
         * get the correct one automatically.
         */
        
        if ((p1.tile_x != p2.tile_x) || (p1.tile_y != p2.tile_y)) {
            p2 = new TileCoordinates(p1.tile_x, p1.tile_y,
                    p2.x + (p2.tile_x - p1.tile_x) * dimx,
                    p2.y + (p2.tile_y - p1.tile_y) * dimy);
        }

        result[0] = p1;
        result[1] = p2;
        return result;
    }

    /**
     * Sets a point on the canvas.
     * 
     * @param p       the coordinates of the point
     * @param colour  the index of the colour
     * @param pixmap  the index of the thickness/bitmap of the point
     */
    public void setPoint(TileCoordinates p, byte colour, int pixmap) {
        int psize = Pixmaps.getSize(pixmap);

        if (psize == 1) {
            setPoint(p, colour);
            return;
        }

        byte[][] pmap = Pixmaps.getPixmap(pixmap);
        int x = p.x - psize / 2;
        for (int k = 0; k < psize; k++) {
            int y = p.y - psize / 2;
            for (int m = 0; m < psize; m++) {
                if (pmap[m][k] != 0) {
                    ps.tile_x = p.tile_x;
                    ps.tile_y = p.tile_y;
                    ps.x = x;
                    ps.y = y;
                    setPoint(ps, colour);
                }
                y = y + 1;
            }
            x = x + 1;
        }
    }

    /**
     * Draws a line on the canvas 
     * 
     * @param p1     starting point of the line
     * @param p2     ending point of the line
     * @param colour colour index for the line
     * @param pixmap index of the thickness/bitmap for each point
     */
    public void drawLine(TileCoordinates p1, TileCoordinates p2, byte colour, int pixmap) {
        TileCoordinates[] points = normalizePair(p1, p2);
        p1 = points[0];
        p2 = points[1];

        // gradient
        double dx = p2.x - p1.x;
        double dy = p2.y - p1.y;
        double a;

        if ((Math.abs(dx) > 1) && (Math.abs(dy / dx) <= 1)) {
            if (dx < 0) {
                TileCoordinates p = p1;
                p1 = p2;
                p2 = p;
                dy = -dy;
                dx = -dx;
            }

            a = dy / dx;
            for (int x = 0; x < dx; x++) {
                int y = (int) (p1.y + a * x);
                setPoint(new TileCoordinates(p1.tile_x, p1.tile_y, p1.x + x, y), colour, pixmap);
            }
        } else if (Math.abs(dy) > 1) {
            if (dy < 0) {
                TileCoordinates p = p1;
                p1 = p2;
                p2 = p;
                dy = -dy;
                dx = -dx;
            }

            a = dx / dy;
            for (int y = 0; y < dy; y++) {
                int x = (int) (p1.x + a * y);
                setPoint(new TileCoordinates(p1.tile_x, p1.tile_y, x, p1.y + y), colour, pixmap);
            }
        } else {
            setPoint(p1, colour, pixmap);
        }
    }
}
