/*
 *  TrackRenderer — Render overlay tiles of GPS tracks for OpenLayers
 *  Copyright (C) 2008 Joerg Reuter <jreuter@yaina.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package de.yaina.trackrenderer;

import java.lang.Math.*;

/**
 * A mercator-projected tiled map to draw positions on
 * 
 * @author Joerg Reuter <jreuter@yaina.de>
 */
public class MapCanvas extends DrawCanvas {

    private int zoom;
    private double previous_lat;
    private double previous_lon;
    private TileCoordinates current_point;

    /**
     * Create a tiled map for a certain zoom level
     * 
     * @param tileDimX horizontal size of a tile
     * @param tileDimY vertical size of a tile
     * @param zoom     zoom level
     */
    public MapCanvas(int tileDimX, int tileDimY, int zoom) {
        super(tileDimX, tileDimY);
        this.zoom = zoom;
        previous_lat = 360;
        previous_lon = 360;
        current_point = null;
    }

    /**
     * Get the zoom level of this map
     * @return zoom level
     */
    public int getZoom() {
        return zoom;
    }

    /**
     * Calculate the longitude of the upper-left corner of a tile
     * 
     * @param x tile number
     * @return longitude
     */
    public double tile_x_to_lon(int x) {
        return (x / Math.pow(2.0, zoom) * 360.0) - 180.0;
    }

    /**
     * Calculate the latitude of the upper-left corner of a tile
     * 
     * @param y
     * @return latitude
     */
    public double tile_y_to_lat(int y) {
        double n = Math.PI - ((2.0 * Math.PI * y) / Math.pow(2.0, zoom));
        return 180.0 / Math.PI * Math.atan(0.5 * (Math.exp(n) - Math.exp(-n)));
    }

    /**
     * Calculate the mercator projected longitude
     * 
     * @param lon longitude
     * @return horizontal value
     */
    public double lon_to_merkator_x(double lon) {
        return (lon + 180.0) / 360.0 * (1 << zoom);
    }
    
    /**
     * Calculate the mercator projected latitude
     * 
     * @param lat latitude
     * @return vertical value
     */
    public double lat_to_merkator_y(double lat) {
        return (1.0 - Math.log(Math.tan(lat * Math.PI / 180.0) +
                1.0 / Math.cos(lat * Math.PI / 180.0)) / Math.PI) / 2.0 * (1 << zoom);
    }

    /**
     * Calculate the vertical tile number from the latitude
     * 
     * @param lat latitude
     * @return tile number
     */
    public int lat_to_tile_y(double lat) {
        return (int) Math.floor(lat_to_merkator_y(lat));
    }
    
    /**
     * Calculate the horizontal tile number from the longitude
     * 
     * @param lon longitude
     * @return tile number
     */
    public int lon_to_tile_x(double lon) {
        return (int) Math.floor(lon_to_merkator_x(lon));
    }

    /**
     * Calculate the horizontal position on the tile
     * 
     * @param lon longitude
     * @return x value
     */
    public int lon_to_x(double lon) {
        double x_merkator = lon_to_merkator_x(lon);
        return (int) ((x_merkator - Math.floor(x_merkator)) * getDimensionX());
    }

    /**
     * Calculate the vertical position on the tile
     * 
     * @param lat latitude
     * @return y value
     */
    public int lat_to_y(double lat) {
        double y_merkator = lat_to_merkator_y(lat);
        return (int) ((y_merkator - Math.floor(y_merkator)) * getDimensionY());
    }

    /**
     * Convert a WGT84 position to a points on the map
     * 
     * @param lat latitude
     * @param lon longitude
     * @return point on the map
     */
    public TileCoordinates positionToPoint(double lat, double lon) {
        int tile_x;
        int tile_y;

        if (current_point != null) {
            if ((previous_lat == lat) && (previous_lon == lon)) {
                return current_point;
            }
            if (previous_lat == lat) {
                tile_x = lon_to_tile_x(lon);
                current_point.tile_x = tile_x;
                current_point.x = lon_to_x(lon);
                return current_point;
            }

            if (previous_lon == lon) {
                tile_y = lat_to_tile_y(lat);
                current_point.tile_y = tile_y;
                current_point.y = lat_to_y(lat);
                return current_point;
            }
        }

        current_point = new TileCoordinates(lon_to_tile_x(lon), lat_to_tile_y(lat), lon_to_x(lon), lat_to_y(lat));
        return current_point;
    }

    /**
     * Set a point on the map
     * @param lat     latitude
     * @param lon     longitude
     * @param colour  colour index
     * @param pixmap  point style
     */
    public void setPoint(double lat, double lon, byte colour, int pixmap) {
        setPoint(positionToPoint(lat, lon), colour, pixmap);
    }

    /**
     * Draw a line on the map. It will be a straight line on the projection,
     * however of course not the shortest distance between the two coordinates
     * 
     * @param lat1      latitude of start coordinate
     * @param lon1      longitude of start coordinate
     * @param lat2      latitude of end coordinate
     * @param lon2      latitude of end coordinate
     * @param colour    colour index
     * @param pixmap    line style
     */
    public void drawLine(double lat1, double lon1, double lat2, double lon2, byte colour, int pixmap) {
        drawLine(positionToPoint(lat1, lon1), positionToPoint(lat2, lon2), colour, pixmap);
    }
}