/*
 *  TrackRenderer — Render overlay tiles of GPS tracks for OpenLayers
 *  Copyright (C) 2008 Joerg Reuter <jreuter@yaina.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package de.yaina.trackrenderer;

import java.util.*;
import java.awt.image.*;

/**
 * A HashMaped collection of tiles. New tiles get created on the fly.
 * 
 * @author Joerg Reuter <jreuter@yaina.de>
 */
public class TileCollection {

    private HashMap Tiles;
    private int old_tilex;
    private int old_tiley;
    private int tileDimensionX;
    private int tileDimensionY;
    private TileRaster current_tile;

    /**
     * Create a new hash map of tiles
     * 
     * @param tileDimX vertical size of a single tile
     * @param tileDimY horizontal size of a single tile
     */
    public TileCollection(int tileDimX, int tileDimY) {
        Tiles = new HashMap();
        tileDimensionX = tileDimX;
        tileDimensionY = tileDimY;
        current_tile = null;
        old_tilex = 0;
        old_tiley = 0;
    }

    /**
     * Return a tile from the map
     * 
     * @param tilex horizontal number of the tile
     * @param tiley vertical number of the tile
     * @return raster image
     */
    public TileRaster getTile(int tilex, int tiley) {
        if ((tilex != old_tilex) || (tiley != old_tiley) || (current_tile == null)) {
            String key = tilex + "/" + tiley;
            current_tile = (TileRaster) Tiles.get(key);
            
            old_tilex = tilex;
            old_tiley = tiley;
        }

        return current_tile;
    }

    /**
     * Get horizontal size of tiles
     * @return horizontal size of tiles
     */
    public int getDimensionX() {
        return tileDimensionX;
    }

    /**
     * Get vertical size of tiles
     * @return vertical size
     */
    public int getDimensionY() {
        return tileDimensionY;
    }

    /**
     * Set a point (a single dot) on the image
     * 
     * @param point  tile and coordinates of the point
     * @param colour colour index of the point
     */
    public void setPoint(TileCoordinates point, byte colour) {
        TileRaster raster = getTile(point.tile_x, point.tile_y);
        if (raster == null) {
            String key = point.tile_x + "/" + point.tile_y;
            raster = new TileRaster(tileDimensionX, tileDimensionY, point.tile_x, point.tile_y);
            if (raster == null) {
                return; // OOM?!

            }
            Tiles.put(key, raster);
        }

        raster.setPoint(point.x, point.y, colour);
    }

    /**
     * Get a point (a single dot) from the image
     * 
     * @param point tile and coordinates of the point
     * @return colour index of the point
     */
    public byte getPoint(TileCoordinates point) {
        TileRaster raster = getTile(point.tile_x, point.tile_y);
        if (raster == null) {
            return 0;
        }
        return raster.getPoint(point.x, point.y);
    }

    /**
     * Return an ARGB image from the raster referenced by tile, suitable
     * for further processing by AWT
     * 
     * @param tilex horizontal tile
     * @param tiley vertical tile
     * @return image
     */
    public BufferedImage getImage(int tilex, int tiley) {
        TileRaster raster = getTile(tilex, tiley);
        if (raster == null) {
            return null;
        }
        return raster.getImage();
    }

    
    /**
     * Return the actual collection of hashed rasters, which is iterable
     * @return collection
     */
    public Collection getCollection() {
        return Tiles.values();
    }
}
