/*
 *  TrackRenderer — Render overlay tiles of GPS tracks for OpenLayers
 *  Copyright (C) 2008 Joerg Reuter <jreuter@yaina.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package de.yaina.trackrenderer;

import java.awt.image.*;

/**
 * Implements a raster image for tiles.
 * Points are stored in a byte array of indexed coulors
 *
 * @author Joerg Reuter <jreuter@yaina.de>
 */
public class TileRaster {

    /* Should be changed to some standard palette */
    private static int[] colours = {
        0x00ffffff,
        0xff000000,
        0xffff0000,
        0xff00ff00,
        0xff0000ff,
        0xffffff00,
        0xffff00ff,
        0xff00ffff,
        0xff808080,
        0xff404040,
        0xff202020
    };
    private int dimensionX;
    private int dimensionY;
    private int tileX;
    private int tileY;
    private byte[] raster;

    /**
     * Create a new image raster
     * 
     * @param dimensionX horizontal size of the pixmap
     * @param dimensionY vertical size of the pixmap
     * @param tileX      number of the X tile (only for stats)
     * @param tileY      number of the Y tile (only for stats)
     */
    public TileRaster(int dimensionX, int dimensionY, int tileX, int tileY) {
        this.dimensionX = dimensionX;
        this.dimensionY = dimensionY;
        this.tileX = tileX;
        this.tileY = tileY;
        raster = new byte[dimensionX * dimensionY];
    }

    /**
     * Set a point on the image map
     * 
     * @param x     x coordinate
     * @param y     y coordinate
     * @param value colour
     */
    public void setPoint(int x, int y, byte value) {
        if ((x < dimensionX) && (y < dimensionY)) {
            raster[x + y * dimensionX] = value;
        }
    }

    /**
     * Read a point from the image map
     * 
     * @param x x coordinate
     * @param y y coordinate
     * @return
     */
    public byte getPoint(int x, int y) {
        if ((x < dimensionX) && (y < dimensionY)) {
            return raster[x + y * dimensionX];
        } else {
            return 0;
        }
    }

    /**
     * Get the horizontal tile number
     * 
     * @return number of the horizontal tile
     */
    public int getTileX() {
        return tileX;
    }

    /**
     * Get the vertical tile number
     * 
     * @return number of the vertical tile
     */
    public int getTileY() {
        return tileY;
    }

    /**
     * Convert the raster to an ARGB image, suitable to be processed by AWT
     * 
     * @return the image
     */
    public BufferedImage getImage() {
        BufferedImage bi = new BufferedImage(dimensionX, dimensionY, BufferedImage.TYPE_INT_ARGB);
        int k;

        for (k = 0; k < dimensionX * dimensionY; k++) {
            int c = raster[k];
            bi.setRGB(k % dimensionX, k / dimensionY, colours[c]);
        }

        return bi;
    }
}
