/*
 *  TrackRenderer — Render overlay tiles of GPS tracks for OpenLayers
 *  Copyright (C) 2008 Joerg Reuter <jreuter@yaina.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package de.yaina.trackrenderer;

import java.io.*;
import java.util.*;
import java.awt.image.*;
import javax.imageio.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * The actual renderer. It reads a GPX file and writes the track on a
 * collection of tiles.
 * 
 * @author Joerg Reuter <jreuter@yaina.de>
 */
public class TrackRender {

    private int zoom;
    private byte colour;
    private int style;
    private String tiledir;
    private MapCanvas canvas;
    private int tileDimX;
    private int tileDimY;
    private static boolean gpx_newsegment;
    private static double current_lat;
    private static double current_lon;
    private static double previous_lat;
    private static double previous_lon;
    public final int DEFAULT_TILE_SIZE = 256;
    public final int DEFAULT_COLOUR = 2;
    public final int DEFAULT_STYLE = Pixmaps.P_LINE3;

    /*
     * Write a tile in PNG format to disk. Create directories as needed.
     */
    
    private void writeTile(TileRaster raster) {
        int x = raster.getTileX();
        int y = raster.getTileY();
        BufferedImage bi = raster.getImage();
        String tile_x_dir = tiledir + "/" + x;

        try {
            File fp = new File(tile_x_dir);
            if (!fp.exists()) {
                fp.mkdirs();
            }
        } catch (Exception e) {
            System.err.println("Cannot create tile directory »" + tile_x_dir + "«");
            e.printStackTrace();
            System.exit(1);
        }

        String filename = tile_x_dir + "/" + y + ".png";
        File fp = new File(filename);
        fp.delete();
        try {
            System.out.println("Writing " + filename);
            ImageIO.write(bi, "png", fp);
        } catch (Exception e) {
            System.err.println("Cannot write tile " + filename);
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * Write the map to disk, as one PNG file per tile
     * 
     * @param canvas the map
     */
    public void writeTiles(MapCanvas canvas) {
        Iterator i = canvas.getCollection().iterator();
        while (i.hasNext()) {
            TileRaster raster = (TileRaster) i.next();
            writeTile(raster);
        }
    }

    /**
     * Set the colour index for the track
     * 
     * @param colour colour index
     */
    public void setColour(byte colour) {
        this.colour = colour;
    }

    /**
     * Return the colour
     * 
     * @return colour index
     */
    public byte getColour() {
        return colour;
    }

    /**
     * Set the point/line style or marker type for this map
     * 
     * @param style point/line style
     */
    public void setStyle(int style) {
        this.style = style;
    }

    /**
     * Return the point/line style for this map
     * @return style
     */
    public int getStyle() {
        return style;
    }

    /**
     * Set the horizontal tile size in pixel
     * 
     * @param dimx horizontal tile size
     */
    public void setTileXSize(int dimx) {
        tileDimX = dimx;
    }

    /**
     * Return the horizontal tile size in pixel
     * @return tile size
     */
    public int getTileXSize() {
        return tileDimX;
    }

    /**
     * Set the vertical tile size in pixel
     * @param dimy vertical tile size
     */
    public void setTileYSize(int dimy) {
        tileDimY = dimy;
    }

    /**
     * Return the vertical tile size in pixel
     * @return tile size
     */
    public int getTileYSize() {
        return tileDimY;
    }

    /**
     * Set the horizontal and vertical tile size to the same value
     * @param size tile size
     */
    public void setTileSize(int size) {
        tileDimX = size;
        tileDimY = size;
    }

    /**
     * Create the canvas to draw on
     * @return reference to canvas
     */
    public MapCanvas createCanvas() {
        canvas = new MapCanvas(tileDimX, tileDimY, zoom);
        return canvas;
    }

    /*
     * SAX handler callbacks for XML processing
     */
    private class xmlHandler extends DefaultHandler {

        @Override
        public void startElement(String uri, String localName, String qName, Attributes attributes) {
            if (qName.equals("trkseg")) {
                gpx_newsegment = true;
            }
            if (qName.equals("trkpt")) {
                String lat = attributes.getValue("lat");
                String lon = attributes.getValue("lon");
                if ((lat == null) || lat.isEmpty() || (lon == null) || lon.isEmpty()) {
                    return;
                }
                current_lat = Double.parseDouble(lat);
                current_lon = Double.parseDouble(lon);
                // System.out.println(gpx_newsegment+" "+current_lat+","+current_lon+" "+previous_lat+","+previous_lon);

                if (!gpx_newsegment) {
                    canvas.drawLine(previous_lat, previous_lon, current_lat, current_lon, colour, style);
                } else {
                    gpx_newsegment = false;
                }
                previous_lat = current_lat;
                previous_lon = current_lon;
            }
        }
    }

    /**
     * Render a track from a GPX file
     * @param gpxfile path of the GPX file
     */
    public void renderMap(String gpxfile) {
        File fp = new File(gpxfile);
        if (!fp.isFile()) {
            System.err.println("File " + gpxfile + " not found or not a plain file.");
            System.exit(1);
        }

        SAXParserFactory parser_factory = SAXParserFactory.newInstance();
        parser_factory.setValidating(false);
        parser_factory.setNamespaceAware(false);

        xmlHandler handler = new xmlHandler();
        gpx_newsegment = true;
        try {
            SAXParser parser = parser_factory.newSAXParser();
            parser.parse(fp, handler);
        } catch (Exception e) {
            System.err.println("Cannot process file " + gpxfile);
            e.printStackTrace();
            System.exit(2);
        }
    }

    /**
     * Create a new renderer
     * @param tiledir where to store the generated tiles
     * @param zoom zoom level of the map
     */
    public TrackRender(String tiledir, int zoom) {
        this.zoom = zoom;
        this.tiledir = tiledir + "/" + zoom;
        tileDimX = DEFAULT_TILE_SIZE;
        tileDimY = DEFAULT_TILE_SIZE;
        colour = DEFAULT_COLOUR;
        style = DEFAULT_STYLE;
    }
}
