/*
 *  TrackRenderer — Render overlay tiles of GPS tracks for OpenLayers
 *  Copyright (C) 2008 Joerg Reuter <jreuter@yaina.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package trackrenderer;

import de.yaina.trackrenderer.*;
import java.io.*;
import java.util.*;
import java.awt.image.*;
import javax.imageio.*;

/**
 * A renderer for track overlays, for OpenLayers tiles
 * @author Joerg Reuter <jreuter@yaina.de>
 */
public class Main {

    private static void drawing_testcase() {
        DrawCanvas tc = new DrawCanvas(32, 32);

        tc.setPoint(new TileCoordinates(3, 3, 16, 16), (byte) 1);
        tc.setPoint(new TileCoordinates(3, 3, 0, 0), (byte) 2);
        tc.setPoint(new TileCoordinates(3, 3, 31, 0), (byte) 2);
        tc.setPoint(new TileCoordinates(3, 3, 0, 31), (byte) 3);
        tc.setPoint(new TileCoordinates(3, 3, 31, 31), (byte) 3);

        tc.setPoint(new TileCoordinates(3, 3, -1, -1), (byte) 4);
        tc.setPoint(new TileCoordinates(3, 3, -33, -33), (byte) 5);

        tc.setPoint(new TileCoordinates(3, 3, 32, 32), (byte) 6);
        tc.setPoint(new TileCoordinates(3, 3, 64, 64), (byte) 7);

        tc.setPoint(new TileCoordinates(3, 3, 16, 16), (byte) 1, Pixmaps.P_MARKER2);
        tc.setPoint(new TileCoordinates(3, 3, 0, 0), (byte) 1, Pixmaps.P_MARKER2);
        tc.setPoint(new TileCoordinates(3, 3, 31, 31), (byte) 1, Pixmaps.P_MARKER2);

        tc.drawLine(
                new TileCoordinates(3, 3, 16, 16),
                new TileCoordinates(4, 4, 16, 16), (byte) 2, Pixmaps.P_LINE3);

        tc.drawLine(
                new TileCoordinates(4, 3, 16, 16),
                new TileCoordinates(3, 2, 4, 4), (byte) 3, Pixmaps.P_LINE1);

        tc.drawLine(
                new TileCoordinates(1, 1, 24, 16),
                new TileCoordinates(1, 1, 8, 4), (byte) 4, Pixmaps.P_LINE1);

        Iterator i = tc.getCollection().iterator();
        while (i.hasNext()) {
            TileRaster r = (TileRaster) i.next();
            BufferedImage bi = r.getImage();
            File fp = new File("test" + r.getTileX() + "-" + r.getTileY() + ".png");
            fp.delete();
            try {
                ImageIO.write(bi, "png", fp);
            } catch (Exception e) {
            }

            System.out.println(r.getTileX() + "," + r.getTileY());
        }
    }

    private static void rendering_testcase() {
        TrackRender r = new TrackRender("/home/jreuter/gps-tracks/debug/trackrender", 10);
        MapCanvas mc = r.createCanvas();

        mc.setPoint(37.616215, -122.385966, (byte) 2, Pixmaps.P_MARKER1); // SFO

        mc.setPoint(36.08, -115.152222, (byte) 2, Pixmaps.P_MARKER1);      // LAS

        mc.setPoint(33.9425, -118.408056, (byte) 2, Pixmaps.P_MARKER1);    // LAX

        mc.drawLine(37.616215, -122.385966, 36.08, -115.152222, (byte) 4, Pixmaps.P_LINE3);

        r.renderMap("/home/jreuter/gps-tracks/debug/test.gpx");
        r.writeTiles(mc);
    }

    private static void rendering_testcase2() {
        TrackRender r = new TrackRender("/home/jreuter/gps-tracks/debug/trackrender", 12);
        MapCanvas mc = r.createCanvas();
        r.renderMap("/home/jreuter/gps-tracks/debug/test2.gpx");
        r.writeTiles(mc);
    }

    private static void usage() {
        System.err.println("usage: trackrender [parameter=value [..]]");
        System.err.println("parameters: linestyle=1..7, colour=1..7, zoom=1..18, tilesize=#, tiledir=path, gpxfile=path");
        System.err.println("            needs at least: zoom, tiledir, gpxfile");
        System.exit(0);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        int linestyle = -1;
        byte colour = (byte) 255;
        int zoom = -1;
        int tilesize = -1;
        String tiledir = null;
        String gpxfile = null;

        int k = 0;
        while (k < args.length) {
            String[] arg = args[k].split("=");
            String what = arg[0];

            if (what.equals("test1")) {
                drawing_testcase();
                return;
            }
            if (what.equals("test2")) {
                rendering_testcase();
                return;
            }
            if (what.equals("test3")) {
                rendering_testcase2();
                return;
            }

            if (arg.length < 2) {
                usage();
            }
            if (what.equals("linestyle")) {
                linestyle = Integer.decode(arg[1]);
            } else if (what.equals("colour")) {
                colour = Byte.decode(arg[1]);
            } else if (what.equals("tilesize")) {
                tilesize = Integer.decode(arg[1]);
            } else if (what.equals("zoom")) {
                zoom = Integer.decode(arg[1]);
            } else if (what.equals("tiledir")) {
                tiledir = arg[1];
            } else if (what.equals("gpxfile")) {
                gpxfile = arg[1];
            }
            k++;
        }

        if ((gpxfile == null) || (tiledir == null) || (zoom < 0)) {
            usage();
        }
        TrackRender r = new TrackRender(tiledir, zoom);
        MapCanvas mc = r.createCanvas();
        if (linestyle > 0) {
            r.setStyle(linestyle);
        }
        if (colour > 0) {
            r.setColour(colour);
        }
        if (tilesize > 0) {
            r.setTileSize(tilesize);
        }
        r.renderMap(gpxfile);
        r.writeTiles(mc);
    }
}
